<?php
/*
 * Photo Resizer Class
 * Copyright (C) 2003-2004, Steve Blinch
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
// Resizes an image proportionally 
define("MOGRIFY","/usr/bin/mogrify");

class PhotoResizer {
	function PhotoResizer() {
		$this->quality = 85;
	}
	
	function resize($filename,$max_width,$max_height,$allow_height=NULL) {
		if (is_null($allow_height)) $allow_height = $max_height;
		if (!is_readable($filename)) {
			$this->error = "Cannot open $filename";
			return false;
		}
		
		$size = @getimagesize($filename);
		if (!$size) {
			$this->error = "Invalid image: $filename";
			return false;
		}
		$width = $size[0];
		$height = $size[1];
		$imagetype = $size[2];
		
		$image_ok = true;
		
		if ($width>$max_width) {
			$height = ($max_width / $width) * $height;
			$width = $max_width;
			$image_ok = false;
		}
		
		if ($height>$allow_height) {
			$width = ($max_height / $height) * $width;
			$height = $max_height;
			$image_ok = false;
		}
		
		// don't process the image if it's already within our size requirements
		if ($image_ok) return true;
		
		$quality = ($imagetype==2?"-quality {$this->quality}":""); // Set quality if it's a JPEG
		
		$width = floor($width);
		$height = floor($height);
		
		$cmd = MOGRIFY." -resize {$width}x{$height} $quality $filename";
        $this->error = trim(`$cmd`);
        
//        echo "[$cmd][{$this->error}]";
//        die;
        
        return ($this->error=="");
	}
	
}
?>