<?php
// Simple File Browser
// Version 1.0.2
// Copyright 2005-2007, Steve Blinch
// http://code.blitzaffe.com
//
// Provides a simple, easy-to-use, aesthetically pleasing file browser for your web
// site.  Includes support for file uploads and batch downloads (by using 'zip' to
// compress a user's file selection "on-the-fly").
//
// Also uses AJAX to provide realtime file information and thumbnails via tooltips.
//
// To install this script, simply edit config.php and follow the instructions.  If
// you don't want everyone to have access to this script, be sure to password protect
// the script using .htaccess files.
//
//
// REQUIREMENTS
//
// This script requires the DirecTemplate template library class which you can
// download at: http://code.blitzaffe.com/pages/phpclasses/files/directemplate_52-2
// Just download DirecTemplate and put it in a subdirectory called "directemplate"
// before using this script.
//
//
// HISTORY
//
// 1.0.2 - Fixed bug when using symlinked folders
//
//
// LICENSE
//
// This script is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This script is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//	
// You should have received a copy of the GNU General Public License along
// with this script; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

function get_cwdparts($cwd) {
	$cwdparts = explode('/',trim($cwd,'/'));
	$cwdpath = '';
	foreach ($cwdparts as $k=>$part) {
		$cwdpath .= $part.'/';
		$cwdparts[$k] = array(
			'path'=>$cwdpath,
			'name'=>$part,
		);
	}
	return $cwdparts;
}

function send_file($filepath,$filename=false,$inline=false) {
	if (!$filename) $filename = basename($filepath);

	header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
	header("Cache-Control: no-store, no-cache, must-revalidate");
	header("Cache-Control: post-check=0, pre-check=0", false);
	header("Pragma: no-cache");
	header('Content-type: ' . get_file_mimetype($filepath));
	if (!$inline) header('Content-Disposition: attachment; filename="'.$filename.'"');
	header("Content-Transfer-Encoding: binary");
	header("Content-Length: ".filesize($filepath));
	
	readfile($filepath);
}


require_once("include.php");

if ($_REQUEST['thumb']) {
	$basepath = realpath(FILE_PATH).'/';
	$filepath = realpath(FILE_PATH.urldecode($_REQUEST['thumb']));
	if (substr($filepath,0,strlen($basepath))!=$basepath) die('Invalid filename');
	
	require_once(dirname(__FILE__)."/class_PhotoResizer.php");
	
	$uid = md5(uniqid(rand(), true));
	$ext = get_ext($filepath);
	$tmpfile = TEMP_PATH."img".$uid.".".$ext;
	
	copy($filepath,$tmpfile);
	
	$w = $_REQUEST['width'] ? $_REQUEST['width'] : 50;
	$h = $_REQUEST['height'] ? $_REQUEST['height'] : 50;
	$a = $_REQUEST['allowed'] ? $_REQUEST['allowed'] : 75;
	
	$resizer = &new PhotoResizer();
	if (!$resizer->resize($tmpfile,$w,$h,$a)) die('Resize error');
	send_file($tmpfile,false,true);
	unlink($tmpfile);
	die;
}

if ($_REQUEST['download']) {
	$basepath = realpath(FILE_PATH).'/';
	$filepath = realpath(FILE_PATH.urldecode($_REQUEST['download']));
	if (substr($filepath,0,strlen($basepath))!=$basepath) die('Invalid filename');

	if (!is_readable($filepath)) die('Cannot access file');

	if ( (!defined('FILE_URI')) || (!FILE_URI) ) {
		send_file($filepath);
	} else {
		$uri = FILE_URI.substr($filepath,strlen($basepath));
		Header('Location: '.$uri);
	}

	die;
}

$basepath = realpath(FILE_PATH).'/';
$cwd = realpath(FILE_PATH.urldecode($_REQUEST['cwd']));

if (substr($cwd,-1)!="/") $cwd .= "/";

if (substr($cwd,0,strlen($basepath))!=$basepath) $cwd = FILE_PATH;
if (!is_dir($cwd)) $cwd = FILE_PATH;

if ($_REQUEST['zipfiles']) {
	$list = "";
	foreach ($_REQUEST['file'] as $name=>$v) {
		$list .= basename($name)."\n";
	}
	$uid = md5(uniqid(rand(), true));
	$listfile = TEMP_PATH."filelist_{$uid}";
	$zipfile = TEMP_PATH."zipfile_{$uid}.zip";
	$fp = fopen($listfile,"w");
	if (!$fp) die("Could not write to temporary directory");
	fwrite($fp,$list);
	fclose($fp);
	
	$oldcwd = getcwd();
	if (!chdir($cwd)) die("Could not access file directory");
	
	exec(sprintf('cat %s | %s -r -@ %s',$listfile,ZIP_BINARY,$zipfile),$output,$retcode);

	unlink($listfile);
	chdir($oldcwd);

	if ($retcode!=0) die("Sorry, an error occurred while creating your ZIP file.  (ZIP returned code: $retcode)");

	send_file($zipfile,basename($cwd).'_'.substr($uid,0,6).'.zip');
	
	unlink($zipfile);
	
	die;
}



$d = dir($cwd);
$files = array();
$dirs = array();
while (false != ($entry = $d->read())) {
	if (substr($entry,0,1)==".") continue;
	if ($entry=="lost+found") continue;
	
	if (is_dir($cwd.$entry)) {
		$dirs[] = array(
			"name"=>$entry,
			"size"=>'Directory',
			"type"=>'folder'
		);
	} else {
		$files[] = array(
			"name"=>$entry,
			"size"=>get_human_size($cwd.$entry),
			"type"=>get_file_type($cwd.$entry),
		);
	}
}
asort($files);
asort($dirs);
$files = array_merge($dirs,$files);
$tpl->assign("files",$files);

$cwd = substr($cwd,strlen($basepath)-1);
$tpl->assign("cwd",$cwd);
$tpl->assign("can_upload",strlen(UPLOAD_PATH));

$tpl->assign("cwdparts",get_cwdparts($cwd));

$tpl->display("header.tpl");
$tpl->display("files.tpl");
$tpl->display("footer.tpl");

?>
